/* Copyright 2016, Michele Santullo
 * This file is part of "keepupnpup".
 *
 * "keepupnpup" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * "keepupnpup" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with "keepupnpup".  If not, see <http://www.gnu.org/licenses/>.
 */

#include "upnp.hpp"
#include "settings.hpp"
#include <iostream>
#include <vector>
#include <string>
#include <ciso646>

namespace {
	bool has_mapping (const std::vector<kuu::Redirection>& parRedirs, const std::string& parAddr, uint16_t parPort, kuu::Protocol parProtocol) {
		for (auto& redir : parRedirs) {
			if (redir.protocol == parProtocol and redir.internal_port == parPort and redir.internal_client == parAddr)
				return true;
		}
		return false;
	}

	void add_bindings (kuu::UPNP& parUpnp, const std::string& parHost, const kuu::RedirectSetting::PortMappingsList& parPortMappings, const std::string& parDesc) {
		const std::vector<kuu::Redirection> redirs; // = upnp.redirections();

		for (auto& mapping : parPortMappings) {
			//std::cout << "Adding " << mapping.port << " --> " << parHost << ":" << mapping.internal_port << " " << mapping.protocol << '\n';
			if (not has_mapping(redirs, parHost, mapping.port, mapping.protocol))
				parUpnp.add_port_mapping(mapping.port, mapping.internal_port, parHost, parDesc, 0, mapping.protocol);
		}
	}

	void runapp() {
		kuu::UPNP upnp;

		kuu::RedirectSetting settings = kuu::load_redirect_settings("keepupnpup.yml", upnp.lanaddr());

		add_bindings(upnp, settings.host, settings.port_mappings, settings.desc);
	}
} //unnamed namespace

int main() {
	try {
		runapp();
	}
	catch (const kuu::Exception& e) {
		std::cerr << e.what() << std::endl;
		return 1;
	}

	return 0;
}

